﻿using System;
using GameUtils.Graphics;
using GameUtils.Graphics.Data;
using GameUtils.Math;

namespace GameUtils.Renderers.Direct2D1_1
{
    internal class Direct2DFactory : Factory
    {
        readonly SharpDX.Direct2D1.Factory1 factory;
        readonly SharpDX.Direct2D1.DeviceContext deviceContext;
        readonly SharpDX.DirectWrite.Factory textFactory;

        internal Direct2DFactory(SharpDX.Direct2D1.Factory1 factory, SharpDX.Direct2D1.DeviceContext deviceContext, SharpDX.DirectWrite.Factory textFactory)
        {
            this.factory = factory;
            this.deviceContext = deviceContext;
            this.textFactory = textFactory;
        }

        protected override SolidColorBrushData CreateSolidColorBrush(Color4 color)
        {
            return new Direct2DSolidColorBrush(this.deviceContext, color);
        }

        protected override LinearGradientBrushData CreateLinearGradientBrush(GradientStop[] stops, Vector2 start, Vector2 end)
        {
            if (stops == null)
                throw new ArgumentNullException("stops");
            if (stops.Length < 2)
                throw new ArgumentException("Es müssen mindestens zwei Verlaufsfarben angegeben sein.", "stops");

            return new Direct2DLinearGradientBrush(this.deviceContext, stops, start, end);
        }

        protected override RadialGradientBrushData CreateRadialGradientBrush(GradientStop[] stops, Vector2 center, float radiusX, float radiusY)
        {
            if (stops == null)
                throw new ArgumentNullException("stops");
            if (stops.Length < 2)
                throw new ArgumentException("Es müssen mindestens zwei Verlaufsfarben angegeben sein.", "stops");

            return new Direct2DRadialGradientBrush(this.deviceContext, stops, center, radiusX, radiusY);
        }

        protected override TextureBrushData CreateTextureBrush(Texture texture)
        {
            if (texture == null)
                throw new ArgumentNullException("texture");

            return new Direct2DTextureBrush(this.deviceContext, texture);
        }

        protected override GeometryData CreateGeometry()
        {
            return new Direct2DGeometry(this.factory);
        }

        protected override FontData CreateFont(string familyName, float size, FontStyle style, FontWeight weight)
        {
            return new Direct2DFont(this.textFactory, familyName, size, style, weight);
        }

        protected override TextureData CreateTexture(string path)
        {
            return new Direct2DTexture(this.deviceContext, path);
        }

        protected override TextureData CreateTexture(System.IO.Stream stream)
        {
            return new Direct2DTexture(this.deviceContext, stream);
        }

        protected override TextFormatData CreateTextFormat()
        {
            return new Direct2DTextFormat();
        }
    }
}
