﻿using GameUtils.Graphics;
using SharpDX;
using Color4 = SharpDX.Color4;
using Ellipse = SharpDX.Direct2D1.Ellipse;
using Vector2 = SharpDX.Vector2;

namespace GameUtils.Renderers.Direct2D1_1
{
    internal static class Direct2DHelper
    {
        public static Color4 GraphicsColorToD2DColor(Graphics.Color4 color)
        {
            return new Color4(color.R, color.G, color.B, color.A);
        }

        public static Graphics.Color4 D2DColorToGraphicsColor(Color4 color)
        {
            return new Graphics.Color4(color.Alpha, color.Red, color.Green, color.Blue);
        }

        public static Vector2 Vector2ToPointF(Math.Vector2 vector)
        {
            return new Vector2(vector.X, vector.Y);
        }

        public static Math.Vector2 PointFToVector2(Vector2 point)
        {
            return new Math.Vector2(point.X, point.Y);
        }

        public static Vector2[] ConvertVectors(Math.Vector2[] vectors)
        {
            var result = new Vector2[vectors.Length];
            for (int i = 0; i < vectors.Length; i++)
                result[i] = new Vector2(vectors[i].X, vectors[i].Y);
            return result;
        }

        public static RectangleF ConvertRectangle(Math.Rectangle rect)
        {
            return new RectangleF(rect.X, rect.Y, rect.Width, rect.Height);
        }

        public static Ellipse GraphicsEllipseToD2DEllipse(Math.Ellipse ellipse)
        {
            return new Ellipse(new Vector2(ellipse.Center.X, ellipse.Center.Y), ellipse.RadiusX, ellipse.RadiusY);
        }

        public static SharpDX.DirectWrite.FontStyle ConvertFontStyle(FontStyle style)
        {
            switch (style)
            {
                case FontStyle.Normal:
                    return SharpDX.DirectWrite.FontStyle.Normal;
                case FontStyle.Italic:
                    return SharpDX.DirectWrite.FontStyle.Italic;
                case FontStyle.Oblique:
                    return SharpDX.DirectWrite.FontStyle.Oblique;
                default:
                    return SharpDX.DirectWrite.FontStyle.Normal;
            }
        }

        public static SharpDX.DirectWrite.FontWeight ConvertFontWeight(FontWeight weigth)
        {
            switch (weigth)
            {
                case FontWeight.Thin:
                    return SharpDX.DirectWrite.FontWeight.Thin;
                case FontWeight.ExtraLight:
                    return SharpDX.DirectWrite.FontWeight.ExtraLight;
                case FontWeight.Light:
                    return SharpDX.DirectWrite.FontWeight.Light;
                case FontWeight.Normal:
                    return SharpDX.DirectWrite.FontWeight.Normal;
                case FontWeight.Medium:
                    return SharpDX.DirectWrite.FontWeight.Medium;
                case FontWeight.SemiBold:
                    return SharpDX.DirectWrite.FontWeight.SemiBold;
                case FontWeight.Bold:
                    return SharpDX.DirectWrite.FontWeight.Bold;
                case FontWeight.ExtraBold:
                    return SharpDX.DirectWrite.FontWeight.ExtraBold;
                case FontWeight.Black:
                    return SharpDX.DirectWrite.FontWeight.Black;
                default:
                    return SharpDX.DirectWrite.FontWeight.Normal;
            }
        }

        public static SharpDX.Direct2D1.GradientStop[] ConvertStops(GradientStop[] stops)
        {
            var result = new SharpDX.Direct2D1.GradientStop[stops.Length];
            for (int i = 0; i < stops.Length; i++)
            {
                var stop = new SharpDX.Direct2D1.GradientStop();
                stop.Color = GraphicsColorToD2DColor(stops[i].Color);
                stop.Position = stops[i].Position;
                result[i] = stop;
            }
            return result;
        }

        public static SharpDX.Direct2D1.InterpolationMode ConvertInterpolationMode(InterpolationMode mode)
        {
            switch (mode)
            {
                case InterpolationMode.NearestNeighbor:
                    return SharpDX.Direct2D1.InterpolationMode.NearestNeighbor;
                case InterpolationMode.Linear:
                    return SharpDX.Direct2D1.InterpolationMode.Linear;
                case InterpolationMode.HighQualityLinear:
                    return SharpDX.Direct2D1.InterpolationMode.MultiSampleLinear;
                case InterpolationMode.Cubic:
                    return SharpDX.Direct2D1.InterpolationMode.Cubic;
                case InterpolationMode.HighQualityCubic:
                    return SharpDX.Direct2D1.InterpolationMode.HighQualityCubic;
                default:
                    return SharpDX.Direct2D1.InterpolationMode.Anisotropic;
            }
        }
    }
}
