﻿using System.Drawing;
using System.Drawing.Drawing2D;
using GameUtils.Graphics;
using GameUtils.Math;
using DashStyle = System.Drawing.Drawing2D.DashStyle;
using InterpolationMode = GameUtils.Graphics.InterpolationMode;

namespace GameUtils.Renderers.Gdi
{
    internal static class GdiHelper
    {
        public static Color ConvertColor(Color4 color)
        {
            return Color.FromArgb((int)(byte.MaxValue * color.A), (int)(byte.MaxValue * color.R), (int)(byte.MaxValue * color.G), (int)(byte.MaxValue * color.B));
        }

        public static PointF Vector2ToPointF(Vector2 vector)
        {
            return new PointF(vector.X, vector.Y);
        }

        public static Vector2 PointFToVector2(PointF point)
        {
            return new Vector2(point.X, point.Y);
        }

        public static PointF[] ConvertVectors(Vector2[] vectors)
        {
            var result = new PointF[vectors.Length];
            for (int i = 0; i < vectors.Length; i++)
                result[i] = new PointF(vectors[i].X, vectors[i].Y);
            return result;
        }

        public static RectangleF ConvertRectangle(Math.Rectangle rect)
        {
            return new RectangleF(rect.X, rect.Y, rect.Width, rect.Height);
        }

        public static RectangleF GetEllipseRect(Ellipse ellipse)
        {
            return new RectangleF(ellipse.Center.X - ellipse.RadiusX, ellipse.Center.Y - ellipse.RadiusY, ellipse.RadiusX * 2, ellipse.RadiusY * 2);
        }

        public static System.Drawing.FontStyle GetFontStyle(Graphics.FontStyle style, FontWeight weight)
        {
            var result = System.Drawing.FontStyle.Regular;
            if (style == Graphics.FontStyle.Italic)
                result = System.Drawing.FontStyle.Italic;
            if (weight == FontWeight.Bold || weight == FontWeight.ExtraBold || weight == FontWeight.SemiBold || weight == FontWeight.Black)
                result |= System.Drawing.FontStyle.Bold;
            return result;
        }

        public static LineCap GetLineCap(CapStyle capStyle)
        {
            switch (capStyle)
            {
                case CapStyle.Round:
                    return LineCap.Round;
                case CapStyle.Square:
                    return LineCap.Square;
                case CapStyle.Triangle:
                    return LineCap.Triangle;
                default:
                    return LineCap.Flat;
            }
        }

        public static DashCap GetDashCap(CapStyle capStyle)
        {
            switch (capStyle)
            {
                case CapStyle.Round:
                    return DashCap.Round;
                case CapStyle.Triangle:
                    return DashCap.Triangle;
                default:
                    return DashCap.Flat;
            }
        }

        public static DashStyle GetDashStyle(Graphics.DashStyle dashStyle)
        {
            switch (dashStyle)
            {
                case Graphics.DashStyle.Custom:
                    return DashStyle.Custom;
                case Graphics.DashStyle.Dash:
                    return DashStyle.Dash;
                case Graphics.DashStyle.DashDot:
                    return DashStyle.DashDot;
                case Graphics.DashStyle.DashDotDot:
                    return DashStyle.DashDotDot;
                case Graphics.DashStyle.Dot:
                    return DashStyle.Dot;
                default:
                    return DashStyle.Solid;
            }
        }

        public static System.Drawing.Drawing2D.InterpolationMode ConvertInterpolytionMode(InterpolationMode mode)
        {
            switch (mode)
            {
                case InterpolationMode.NearestNeighbor:
                    return System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
                case InterpolationMode.Linear:
                    return System.Drawing.Drawing2D.InterpolationMode.Bilinear;
                case InterpolationMode.HighQualityLinear:
                    return System.Drawing.Drawing2D.InterpolationMode.HighQualityBilinear;
                case InterpolationMode.Cubic:
                    return System.Drawing.Drawing2D.InterpolationMode.Bicubic;
                case InterpolationMode.HighQualityCubic:
                    return System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                default:
                    return System.Drawing.Drawing2D.InterpolationMode.Default;
            }
        }
    }
}
