﻿using System;
using System.IO;
using GameUtils.Graphics.Data;
using GameUtils.Math;

namespace GameUtils.Graphics
{
    public class Texture : Resource<TextureData>
    {
        readonly Stream stream;
        readonly string path;

        /// <summary>
        /// The width of this texture in pixels.
        /// </summary>
        public int Width
        {
            get { return IsCreated ? Data.Width : 0; }
        }

        /// <summary>
        /// The height of this texture in pixels.
        /// </summary>
        public int Height
        {
            get { return IsCreated ? Data.Height : 0; }
        }

        /// <summary>
        /// The size of this texture in pixels.
        /// </summary>
        public Vector2 Size
        {
            get { return Data != null ? new Vector2(Data.Width, Data.Height) : default(Vector2); }
        }

        /// <summary>
        /// Creates a new texture.
        /// </summary>
        public Texture(Stream stream)
        {
            if (stream == null)
                throw new ArgumentNullException("stream");

            this.stream = stream;
            path = null;
        }

        /// <summary>
        /// Creates a new texture.
        /// </summary>
        public Texture(string path)
        {
            if (path == null)
                throw new ArgumentNullException("path");

            stream = null;
            this.path = path;
        }

        protected override void ApplyChanges()
        { }

        protected override TextureData CreateData(Factory factory)
        {
            return stream != null ? factory.CreateTexture(stream) : factory.CreateTexture(path);
        }
    }
}
